# -*- coding: iso-8859-1 -*-

from simulation import Controller

class FuzzyController(Controller.Controller):
    """Fuzzy controller."""

    def __init__(self):
        import fuzzy.storage.fcl.Reader
        self.system = fuzzy.storage.fcl.Reader.Reader().load_from_file("container_crane.fcl")

        self.system.description = """This fuzzy system is to control the container crane."""

    def calculate(self,input={},output={'a':0.0}):
        my_input = {
            "distance":input["X"],
            "angle":-(input["Phi"]-270.),
        }
        my_output = { "power": 0.0 }
        self.system.calculate(my_input,my_output)
        output['a'] = -my_output["power"]/1.0
        # hold old value if no results in next calculations
        #self.system.variables['power'].defuzzify.failsafe = output['power']
        #print input,output
        #print my_input,my_output
        return output

    def createDoc(self,directory):
        from fuzzy.doc.plot.gnuplot import doc
        d = doc.Doc(directory)
        d.createDoc(self.system)
        d.overscan=0
        d.create3DPlot(self.system,"distance","angle","power",{})

    def createDot(self,directory):
        import fuzzy.doc.structure.dot.dot
        import subprocess
        for name,rule in self.system.rules.items():
            cmd = "dot -T png -o '%s/Rule %s.png'" % (directory,name)
            f = subprocess.Popen(cmd, shell=True, bufsize=32768, stdin=subprocess.PIPE).stdin
            fuzzy.doc.structure.dot.dot.print_header(f,"XXX")
            fuzzy.doc.structure.dot.dot.print_dot(rule,f,self.system,"")
            fuzzy.doc.structure.dot.dot.print_footer(f)
        cmd = "dot -T png -o '%s/System.png'" % directory
        f = subprocess.Popen(cmd, shell=True, bufsize=32768, stdin=subprocess.PIPE).stdin
        fuzzy.doc.structure.dot.dot.printDot(self.system,f)
